<?php
	
class Soliloquy_Instagram_Common{
	
    /**
     * Holds the class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public static $instance;

    /**
     * Path to the file.
     *
     * @since 2.2.0
     *
     * @var string
     */
    public $file = __FILE__;

    /**
     * Holds the base class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public $base;
    
    /**
     * Primary class constructor.
     *
     * @since 2.2.0
     */	
     function __construct(){
	     
	     $this->base = Soliloquy_Instagram::get_instance();
     }
     
	/**
	 * Returns the available Instagram query types.
	 *
	 * @since 2.1.2
	 *
	 * @return array Array of Instagram query types.
	 */
	function types() {
	
	    $types = array(
	        array(
	            'value' => 'users_self_media_recent',
	            'name'  => esc_attr__( 'My Instagram Photos', 'soliloquy-instagram' )
	        ),
//	        array(
//	            'value' => 'users_self_media_liked',
//	            'name'  => esc_attr__( 'Instagram Photos I\'ve Liked', 'soliloquy-instagram' )
//	        ),
//	        array(
//	            'value' => 'tags_tag_media_recent',
//	            'name'  => esc_attr__( 'Instagram Photos by Tag', 'soliloquy-instagram' )
//	        ),
	    );
	
	    return apply_filters( 'soliloquy_instagram_types', $types );
	
	}

	/**
	 * Returns the available Instagram image resolutions.
	 *
	 * @since 1.0.0
	 *
	 * @return array Array of Instagram image resolutions.
	 */
	function resolutions() {
	
	    $resolutions = array(
	        array(
	            'value' => 'thumbnail',
	            'name'  => esc_attr__( 'Thumbnail (150x150)', 'soliloquy-instagram' )
	        ),
	        array(
	            'value' => 'low_resolution',
	            'name'  => esc_attr__( 'Low Resolution (306x306)', 'soliloquy-instagram' )
	        ),
	        array(
	            'value' => 'standard_resolution',
	            'name'  => esc_attr__( 'Standard Resolution (640x640)', 'soliloquy-instagram' )
	        )
	    );
	
	    return apply_filters( 'soliloquy_instagram_resolutions', $resolutions );
	
	}

	/**
	 * Returns Instagram auth data.
	 *
	 * @since 1.0.0
	 *
	 * @return string|bool Access token on success, false on failure.
	 */
	function get_auth() {
	
	    return get_option( 'soliloquy_instagram' );
	
	}

	/**
	 * Flag to check for Instagram auth data.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True if auth data is available, false otherwise.
	 */
	function has_auth() {
	
	    return isset( $_GET['soliloquy-instagram'] ) && ! empty( $_GET['soliloquy-instagram'] );
	
	}

	/**
	 * Flushes the Instagram data caches on save.
	 *
	 * @since 1.0.0
	 *
	 * @param int $post_id The current post ID.
	 * @param string $slug The current slider slug.
	 */
	function instagram_flush_caches( $post_id, $slug ) {
	
	    delete_transient( '_sol_instagram_' . $post_id );
	    delete_transient( '_sol_instagram_' . $slug );
	
	}

	/**
	 * Flag to check for removing Instagram auth data.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True if auth data is being removed, false otherwise.
	 */
	function removing_auth() {
	
	    return isset( $_GET['soliloquy-instagram-remove'] ) && $_GET['soliloquy-instagram-remove'];
	
	}
	
	/**
	 * Sanitizes a caption and removes emoticons.
	 *
	 * @since 1.0.0
	 *
	 * @param string $caption  An image caption.
	 * @return string $caption Sanizited image caption.
	 */
	function clean_caption( $caption ) {
	
	    return preg_replace( '/([0-9|#][\x{20E3}])|[\x{00ae}|\x{00a9}|\x{203C}|\x{2047}|\x{2048}|\x{2049}|\x{3030}|\x{303D}|\x{2139}|\x{2122}|\x{3297}|\x{3299}][\x{FE00}-\x{FEFF}]?|[\x{2190}-\x{21FF}][\x{FE00}-\x{FEFF}]?|[\x{2300}-\x{23FF}][\x{FE00}-\x{FEFF}]?|[\x{2460}-\x{24FF}][\x{FE00}-\x{FEFF}]?|[\x{25A0}-\x{25FF}][\x{FE00}-\x{FEFF}]?|[\x{2600}-\x{27BF}][\x{FE00}-\x{FEFF}]?|[\x{2900}-\x{297F}][\x{FE00}-\x{FEFF}]?|[\x{2B00}-\x{2BF0}][\x{FE00}-\x{FEFF}]?|[\x{1F000}-\x{1F6FF}][\x{FE00}-\x{FEFF}]?/u', '', $caption );
	
	}
		
    /**
     * Returns the singleton instance of the class.
     *
     * @since 2.2.0
     *
     * @return object The Soliloquy_Common object.
     */
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Soliloquy_Instagram_Common ) ) {
            self::$instance = new Soliloquy_Instagram_Common();
        }

        return self::$instance;

    }	
}

$soliloquy_instagram_common = Soliloquy_Instagram_Common::get_instance();