<?php

class Soliloquy_Instagram_Metaboxes{

    /**
     * Holds the class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public static $instance;

    /**
     * Path to the file.
     *
     * @since 2.2.0
     *
     * @var string
     */
    public $file = __FILE__;

    /**
     * Holds the base class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public $base;
    /**
     * Primary class constructor.
     *
     * @since 2.2.0
     */
    public function __construct() {

    	// Get base instance
    	$this->base = Soliloquy_Instagram::get_instance();
  	    
	    // Run a verifcation and removal check for Instagram auth data.
	    $this->verify_auth();
	    $this->remove_auth();
	    
		// Add hooks and filters.
	    add_filter( 'soliloquy_defaults', array( $this, 'defaults' ), 10, 2 );
		add_action( 'soliloquy_metabox_scripts', array( $this, 'scripts' ) );
		add_filter( 'soliloquy_slider_types', array( $this, 'type' ) );
		add_action( 'soliloquy_display_instagram', array( $this, 'settings' ) );
		add_filter( 'soliloquy_save_settings', array( $this, 'save' ), 10, 2 );
 //   add_action( 'soliloquy_flush_caches', 'soliloquy_instagram_flush_caches', 10, 2 );  
    }

	/**
	 * Checks for and saves Instagram auth data.
	 *
	 * @since 1.0.0
	 *
	 * @retun null Return early if no auth is found.
	 */
	function verify_auth() {
		
		$common = Soliloquy_Instagram_Common::get_instance();
	    
	    // Return early if no auth data is present.
	    if ( ! $common->has_auth() ) {
	        return;
	    }
	
	    // Save the auth access token.
	    $data = json_decode( stripslashes_deep( $_GET['soliloquy-instagram'] ) );
	    if ( empty( $data->access_token ) || empty( $data->user->id ) ) {
	        return;
	    }
	
	    // Update the option with the Instagram access token and user ID.
	    $auth          = $common->get_auth();
	    $auth['token'] = $data->access_token;
	    $auth['id']    = $data->user->id;
	    update_option( 'soliloquy_instagram', $auth );
	
	}
	
	/**
	 * Removes Instagram auth data from the site.
	 *
	 * @since 1.0.0
	 *
	 * @retun null Return early if no auth is found.
	 */
	function remove_auth() {
		
		$common = Soliloquy_Instagram_Common::get_instance();

	    // Return early if not removing the auth data.
	    if ( ! $common->removing_auth() ) {
	        return;
	    }
	
	    // Return early if no auth data is present.
	    $auth = $common->get_auth();
	    if ( ! $auth ) {
	        return;
	    }
	
	    // Update the option to remove the auth.
	    update_option( 'soliloquy_instagram', $this->base->default_options() );
	
	}
	
	/**
	 * Enqueues scripts for metabox
	 *
	 * @since 2.1.2
	 */
	function scripts() {
	
	    wp_enqueue_script( $this->base->plugin_slug . '-metabox-script', plugins_url( 'assets/js/metabox.js', $this->base->file ), array( 'jquery' ), $this->base->version, true );
	
	}
		
	/**
	 * Applies a default to the addon setting.
	 *
	 * @since 1.0.0
	 *
	 * @param array $defaults  Array of default config values.
	 * @param int $post_id     The current post ID.
	 * @return array $defaults Amended array of default config values.
	 */
	function defaults( $defaults, $post_id ) {
	
	    $defaults['instagram_type']    = 'users_self_media_recent';
	    $defaults['instagram_tag']     = '';
	    $defaults['instagram_number']  = 5;
	    $defaults['instagram_res']     = 'standard_resolution';
	    $defaults['instagram_link']    = 0;
	    $defaults['instagram_caption'] = 1;
	    $defaults['instagram_random']  = 0;
	    $defaults['instagram_cache']   = 1;
	    return $defaults;
	
	}    
	
	/**
	 * Adds the "Instagram" slider type to the list of available options.
	 *
	 * @since 1.0.0
	 *
	 * @param array $types  Types of sliders to select.
	 * @return array $types Amended types of sliders to select.
	 */
	function type( $types ) {
	
	    $types['instagram'] = esc_attr__( 'Instagram', 'soliloquy-instagram' );
	    return $types;
	
	}
	


	/**
	 * Callback for displaying the UI for setting instagram options.
	 *
	 * @since 1.0.0
	 *
	 * @param object $post The current post object.
	 */
	function settings( $post ) {
		
		global $screen;
		
	    $instance = Soliloquy_Metaboxes::get_instance();
	    $common   = Soliloquy_Instagram_Common::get_instance();
	    $auth     = $common->get_auth();
	
	    if ( empty( $auth['token'] ) ) : 
			if ( 'auto-draft' == $post->post_status ) {
				$connect_url = 'post-new.php?post_type=soliloquy' ;
			} else {
				$connect_url = 'post.php?post=' . $post->ID;
			}  
	    ?>
	
	    <div class="soliloquy-external-req">
		    
	        <h2><?php _e('Instagram Authorization Setup', 'soliloquy'); ?></h2>
	        <p><?php _e( 'Before you can create Instagram sliders, you need to authenticate Soliloquy with your Instagram account.', 'soliloquy-instagram' ); ?></p>
			<p><a href="<?php echo add_query_arg( array( 'client_id' => 'c85ddd3230e1419ea8c9a62169cae60b', 'response_type' => 'code', 'redirect_uri' => 'http://soliloquywp.com/instagram?return_to=' . urlencode( admin_url( $connect_url ) ) ), 'https://api.instagram.com/oauth/authorize/' ); ?>" class="button button-soliloquy"><?php _e( 'Click here to Authenticate Soliloquy with Instagram', 'soliloquy-instagram' ); ?></a></p>
		    
	    </div>
	    
	    <?php else : ?>
	    
	    <div id="soliloquy-instagram">
		    <div class="soliloquy-config-header">
	        	<h2 class="soliloquy-intro"><?php _e( 'The settings below adjust the Instagram settings for the slider.', 'soliloquy-instagram' ); ?></h2>
				<p class="soliloquy-help"><?php _e( 'Need some help?', 'soliloquy-thumbnails' ); ?><a href="http://soliloquywp.com/docs/instagram-addon" target="_blank"><?php _e( ' Watch a video on how to setup your slider configuration', 'soliloquy-schedule' ); ?></a></p>
			</div>	    
	    
	        <table class="form-table">
	    
	            <tbody>
	                <tr id="soliloquy-config-instagram-type-box">
	                    <th scope="row">
	                        <label for="soliloquy-config-instagram-type"><?php _e( 'Feed Type', 'soliloquy-instagram' ); ?></label>
	                    </th>
	                    <td>
		                    <div class="soliloquy-select">
	                        <select id="soliloquy-config-instagram-type" name="_soliloquy[instagram_type]" class="soliloquy-chosen" data-soliloquy-chosen-options='{ "disable_search":"true", "width": "100%" }'>
	                            <?php foreach ( (array) $common->types() as $i => $data ) : ?>
	                                <option value="<?php echo $data['value']; ?>"<?php selected( $data['value'], $instance->get_config( 'instagram_type', $instance->get_config_default( 'instagram_type' ) ) ); ?>><?php echo $data['name']; ?></option>
	                            <?php endforeach; ?>
	                        </select>
		                    </div>
	                        <p class="description"><?php _e( 'The type of images to pull from Instagram.', 'soliloquy-instagram' ); ?></p>
	                    </td>
	                </tr>
<!--	                <tr id="soliloquy-config-instagram-tag-box">
	                    <th scope="row">
	                        <label for="soliloquy-config-instagram-tag"><?php _e( 'Tag', 'soliloquy-instagram' ); ?></label>
	                    </th>
	                    <td>
	                        <input id="soliloquy-config-instagram-tag" type="text" name="_soliloquy[instagram_tag]" value="<?php echo $instance->get_config( 'instagram_tag', $instance->get_config_default( 'instagram_tag' ) ); ?>" />
	                        <p class="description"><?php _e( 'Pulls images that match the given tag.', 'soliloquy-instagram' ); ?></p>
	                    </td>
	                </tr>-->
	                <tr id="soliloquy-config-instagram-number-box">
	                    <th scope="row">
	                        <label for="soliloquy-config-instagram-number"><?php _e( 'Number of Instagram Photos', 'soliloquy-instagram' ); ?></label>
	                    </th>
	                    <td>
	                        <input id="soliloquy-config-instagram-number" type="number" name="_soliloquy[instagram_number]" value="<?php echo $instance->get_config( 'instagram_number', $instance->get_config_default( 'instagram_number' ) ); ?>" />
	                        <p class="description"><?php _e( 'The number of images to pull from your Instagram feed.', 'soliloquy-instagram' ); ?></p>
	                    </td>
	                </tr>
	                <tr id="soliloquy-config-instagram-res-box">
	                        <th scope="row">
	                            <label for="soliloquy-config-instagram-res"><?php _e( 'Image Resolution', 'soliloquy' ); ?></label>
	                        </th>
	                        <td>
		                        <div class="soliloquy-select">
	                            <select id="soliloquy-config-instagram-res" name="_soliloquy[instagram_res]" class="soliloquy-chosen" data-soliloquy-chosen-options='{ "disable_search":"true", "width": "100%" }'>
	                                <?php foreach ( (array) $common->resolutions() as $i => $data ) : ?>
	                                    <option value="<?php echo $data['value']; ?>"<?php selected( $data['value'], $instance->get_config( 'instagram_res', $instance->get_config_default( 'instagram_res' ) ) ); ?>><?php echo $data['name']; ?></option>
	                                <?php endforeach; ?>
	                            </select>
		                        </div>
	                            <p class="description"><?php _e( 'Determines the image resolution and size to use from Instagram.', 'soliloquy' ); ?></p>
	                        </td>
	                    </tr>
	                <tr id="soliloquy-config-instagram-link-box">
	                    <th scope="row">
	                        <label for="soliloquy-config-instagram-link"><?php _e( 'Link to Instagram Location?', 'soliloquy' ); ?></label>
	                    </th>
	                    <td>
	                        <input id="soliloquy-config-instagram-link" type="checkbox" name="_soliloquy[instagram_link]" value="<?php echo $instance->get_config( 'instagram_link', $instance->get_config_default( 'instagram_link' ) ); ?>" <?php checked( $instance->get_config( 'instagram_link', $instance->get_config_default( 'instagram_link' ) ), 1 ); ?> />
	                        <span class="description"><?php _e( 'Links the photo to its original location on Instagram.', 'soliloquy' ); ?></span>
	                    </td>
	                </tr>
	                <tr id="soliloquy-config-instagram-caption-box">
	                    <th scope="row">
	                        <label for="soliloquy-config-instagram-caption"><?php _e( 'Use Photo Caption?', 'soliloquy' ); ?></label>
	                    </th>
	                    <td>
	                        <input id="soliloquy-config-instagram-caption" type="checkbox" name="_soliloquy[instagram_caption]" value="<?php echo $instance->get_config( 'instagram_caption', $instance->get_config_default( 'instagram_caption' ) ); ?>" <?php checked( $instance->get_config( 'instagram_caption', $instance->get_config_default( 'instagram_caption' ) ), 1 ); ?> />
	                        <span class="description"><?php _e( 'Displays the photo caption from Instagram on the slide.', 'soliloquy' ); ?></span>
	                    </td>
	                </tr>
	                <tr id="soliloquy-config-instagram-cache-box">
	                    <th scope="row">
	                        <label for="soliloquy-config-instagram-cache"><?php _e( 'Cache Data from Instagram?', 'soliloquy' ); ?></label>
	                    </th>
	                    <td>
	                        <input id="soliloquy-config-instagram-cache" type="checkbox" name="_soliloquy[instagram_cache]" value="<?php echo $instance->get_config( 'instagram_cache', $instance->get_config_default( 'instagram_cache' ) ); ?>" <?php checked( $instance->get_config( 'instagram_cache', $instance->get_config_default( 'instagram_cache' ) ), 1 ); ?> />
	                        <span class="description"><?php _e( 'Caches the data from Instagram to improve performance (recommended).', 'soliloquy' ); ?></span>
	                    </td>
	                </tr>
	                <?php do_action( 'soliloquy_instagram_box', $post ); ?>
	            </tbody>
	        </table>
	        
	    </div>
	    <?php endif;
	
	}
	
	/**
	 * Saves the addon settings.
	 *
	 * @since 1.0.0
	 *
	 * @param array $settings  Array of settings to be saved.
	 * @param int $post_id     The current post ID.
	 * @return array $settings Amended array of settings to be saved.
	 */
	function save( $settings, $post_id ) {
	
	    // If not saving a featured content slider, do nothing.
	    if ( ! isset( $_POST['_soliloquy']['type_instagram'] ) ) {
	        return $settings;
	    }
	
	    // Save the settings.
	    $settings['config']['instagram_type']    = esc_attr( $_POST['_soliloquy']['instagram_type'] );
	    $settings['config']['instagram_tag']     = sanitize_text_field( $_POST['_soliloquy']['instagram_tag'] );
	    $settings['config']['instagram_number']  = absint( $_POST['_soliloquy']['instagram_number'] );
	    $settings['config']['instagram_res']     = esc_attr( $_POST['_soliloquy']['instagram_res'] );
	    $settings['config']['instagram_link']    = isset( $_POST['_soliloquy']['instagram_link'] ) ? 1 : 0;
	    $settings['config']['instagram_caption'] = isset( $_POST['_soliloquy']['instagram_caption'] ) ? 1 : 0;
	    $settings['config']['instagram_random']  = isset( $_POST['_soliloquy']['instagram_random'] ) ? 1 : 0;
	    $settings['config']['instagram_cache']   = isset( $_POST['_soliloquy']['instagram_cache'] ) ? 1 : 0;
	    return $settings;
	
	}	
    /**
     * Returns the singleton instance of the class.
     *
     * @since 2.2.0
     *
     * @return object The Soliloquy_Lightbox_Mextabox object.
     */
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Soliloquy_Instagram_Metaboxes ) ) {
            self::$instance = new Soliloquy_Instagram_Metaboxes();
        }

        return self::$instance;

    }
    	
}

$soliloquy_instagram_metaboxes = Soliloquy_Instagram_Metaboxes::get_instance();