<?php
	
class Soliloquy_Instagram_Settings{
	
    /**
     * Holds the class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public static $instance;

    /**
     * Path to the file.
     *
     * @since 2.2.0
     *
     * @var string
     */
    public $file = __FILE__;

    /**
     * Holds the base class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public $base;
    
	function __construct(){
		add_filter( 'soliloquy_settings_tab_nav', array( $this, 'settings_tab' ) );
		add_action( 'soliloquy_tab_settings_instagram', array( $this, 'settings_output' ) );	
	}
	/**
	 * Adds a custom settings tab to the Settings screen.
	 *
	 * @since 1.0.0
	 *
	 * @param array $tabs  Array of default settings tabs.
	 * @return array $tabs Amended array of default settings tabs.
	 */
	function settings_tab( $tabs ) {
	
	    $tabs['instagram'] = esc_attr__( 'Instagram', 'soliloquy-instagram' );
	    return $tabs;
	
	}
	
	/**
	 * Outputs the custom settings HTML.
	 *
	 * @since 1.0.0
	 */
	function settings_output() {
		$common = Soliloquy_Instagram_Common::get_instance();
	    ?>
	    <div id="soliloquy-settings-instagram">
	        <?php $auth = $common->get_auth(); if ( ! empty( $auth['token'] ) ) : ?>
	  	    <div class="soliloquy-external-req-active">
	  			<h2><?php esc_html_e('Success!', 'soliloquy'); ?></h2>
	            <p><?php esc_html_e( 'Your Instagram account has been authenticated for use with Soliloquy!', 'soliloquy-instagram' ); ?></p>
				<p><a href="<?php echo add_query_arg( array( 'post_type' => 'soliloquy', 'soliloquy-instagram-remove' => 'true', 'page' => 'soliloquy-settings#soliloquy-tab-instagram' ), admin_url( 'edit.php' ) ); ?>" class="button button-soliloquy-secondary soliloquy-oauth-remove"><?php esc_html_e( 'Click Here to Remove Instagram Authentication from Soliloquy', 'soliloquy' ); ?></a></p>
	  	    
	  	    </div>
	  	    
	        <?php else : ?>
		    <div class="soliloquy-external-req">
		        <h2><?php esc_html_e('Instagram Authorization Setup', 'soliloquy'); ?></h2>
		        <p><?php esc_html_e( 'Before you can create Instagram sliders, you need to authenticate Soliloquy with your Instagram account.', 'soliloquy-instagram' ); ?></p>
				<p><a href="<?php echo add_query_arg( array( 'client_id' => 'c85ddd3230e1419ea8c9a62169cae60b', 'response_type' => 'code', 'redirect_uri' => 'http://soliloquywp.com/instagram?return_to=' . urlencode( admin_url( 'edit.php?post_type=soliloquy&page=soliloquy-settings' ) ) ), 'https://api.instagram.com/oauth/authorize/' ); ?>" class="button button-soliloquy"><?php esc_html_e( 'Click here to Authenticate Soliloquy with Instagram', 'soliloquy-instagram' ); ?></a></p>
			    
		    </div>        
	        <?php endif; ?>
	        <?php do_action( 'soliloquy_settings_instagram_box' ); ?>
	    </div>
	    <?php
	
	}

    /**
     * Returns the singleton instance of the class.
     *
     * @since 2.2.0
     *
     * @return object The Soliloquy_Lightbox_Mextabox object.
     */
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Soliloquy_Instagram_Settings ) ) {
            self::$instance = new Soliloquy_Instagram_Settings();
        }

        return self::$instance;

    }
    	

}

$soliloquy_instagram_settings = Soliloquy_Instagram_Settings::get_instance();