<?php
	
	
class Soliloquy_Instagram_Shortcode{
    /**
     * Holds the class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public static $instance;

    /**
     * Path to the file.
     *
     * @since 2.2.0
     *
     * @var string
     */
    public $file = __FILE__;

    /**
     * Holds the base class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public $base;

    /**
     * Primary class constructor.
     *
     * @since 2.2.0
     */	
	function __construct(){
		
		$this->base = Soliloquy_Instagram::get_instance();
		
		add_filter( 'soliloquy_pre_data', array( $this, 'data' ), 10, 2 );
		add_filter( 'soliloquy_output_classes', array( $this, 'output_class' ), 10, 2 );	  
    		
	}
	
	/**
	 * Filters the data to pull images from Instagram for Instagram sliders.
	 *
	 * @since 1.0.0
	 *
	 * @param array $data  Array of slider data.
	 * @param int $id      The slider ID.
	 * @return array $data Amended array of slider data.
	 */
	function data( $data, $id ) {
	
	    // Return early if not an Instagram slider.
	    $instance = Soliloquy_Shortcode::get_instance();
	    if ( 'instagram' !== $instance->get_config( 'type', $data ) ) {
	        return $data;
	    }
	
	    // Grab the Instagram data.
	    $instagram = $instance->get_config( 'instagram_cache', $data ) ? $this->get_data( $id, $data ) : $this->_get_data( $id, $data );
	    if ( ! $instagram ) {
	        return $data;
	    }
	
	    // Now that we have the proper data, insert the Instagram data into the slider data.
	    $data = $this->insert_data( $data, $instagram );
	
	    // Return the modified data.
	    return apply_filters( 'soliloquy_instagram_data', $data, $id );
	
	}
	
	/**
	 * Grabs the appropriate data from Instagram.
	 *
	 * @since 1.0.0
	 *
	 * @param mixed $id   The current slider ID.
	 * @param array $data Array of slider data.
	 * @return array|bool Array of data on success, false on failure.
	 */
	function get_data( $id, $data ) {
	
	    // Attempt to return the transient first, otherwise generate the new query to retrieve the data.
	    if ( false === ( $instagram_data = get_transient( '_sol_instagram_' . $id ) ) ) {
	        $instagram_data = $this->_get_data( $id, $data );
	        if ( $instagram_data ) {
	            set_transient( '_sol_instagram_' . $id, maybe_serialize( $instagram_data ), DAY_IN_SECONDS );
	        }
	    }
	
	    // Return the slider data.
	    return maybe_unserialize( $instagram_data );
	
	}
	
	/**
	 * Grabs the appropriate data from Instagram if the transient doesn't exist.
	 *
	 * @since 1.0.0
	 *
	 * @param mixed $id   The current slider ID.
	 * @param array $data Array of slider data.
	 * @return array|bool Array of data on success, false on failure.
	 */
	function _get_data( $id, $data ) {
		$common = Soliloquy_Instagram_Common::get_instance();
	    // Grab the Instagram auth data.
	    $auth = $common->get_auth();
	    if ( empty( $auth['token'] ) || empty( $auth['id'] ) ) {
	        return false;
	    }
	
	    // Ping Instagram to retrieve the proper data.
	    $instance = Soliloquy_Shortcode::get_instance();
	    switch ( $instance->get_config( 'instagram_type', $data ) ) {
	        case 'users_self_media_recent':
	        default:
	            $response = wp_remote_get( esc_url_raw( 'https://api.instagram.com/v1/users/' . $auth['id'] . '/media/recent/?access_token=' . $auth['token'] . '&count=' . $instance->get_config( 'instagram_number', $data ) ) );
	            break;
	        case 'users_self_media_liked':
	            $response = wp_remote_get( esc_url_raw( 'https://api.instagram.com/v1/users/self/media/liked/?access_token=' . $auth['token'] . '&count=' . $instance->get_config( 'instagram_number', $data ) ) );
	            break;
	        case 'tags_tag_media_recent':
	            $response = wp_remote_get( esc_url_raw( 'https://api.instagram.com/v1/tags/' . urlencode( $instance->get_config( 'instagram_tag', $data ) ) . '/media/recent/?access_token=' . $auth['token'] . '&count=' . $instance->get_config( 'instagram_number', $data ) ) );
	            break;
	    }
	    
	    // If there is an error with the request, return false.
	    if ( is_wp_error( $response ) ) {
	        return false;
	    }
	
	    // Parse and decode the response body. If there is an error or no response body, return false
	    $body = json_decode( wp_remote_retrieve_body( $response ), true );
	    if ( is_null( $body ) || empty( $body['data'] ) ) {
	        return false;
	    }
	
	    // Loop through the response data and remove any emoticons that can't be stored in the DB.
	    $sanitized_data = array();
	    $res            = $instance->get_config( 'instagram_res', $data );
	    foreach ( $body['data'] as $i => $image ) {
	
	        $sanitized_data[$i]['id']      = $image['id'];
	        $sanitized_data[$i]['status']  = 'active';
	        $sanitized_data[$i]['src']     = ! empty( $image['images'][$res]['url'] ) ? esc_url( $image['images'][$res]['url'] ) : '';
	        $sanitized_data[$i]['caption'] = $instance->get_config( 'instagram_caption', $data ) && ! empty( $image['caption']['text'] ) ? esc_attr( $common->clean_caption( $image['caption']['text'] ) ) : '';
	        $sanitized_data[$i]['link']    = $instance->get_config( 'instagram_link', $data ) && ! empty( $image['link'] ) ? esc_url( $image['link'] ) : '';
	        $sanitized_data[$i]['type']    = 'image';
	       // $sanitized_data[$i]['ig_likes']    = absint( $image['likes']['count']);
	       // $sanitized_data[$i]['ig_comments']    = absint($image['comments']['count']);
	    }
	
	    // Return the sanitized data.
	    return apply_filters( 'soliloquy_instagram_sanitized_data', $sanitized_data, $body['data'], $id, $data );
	
	}
		
	/**
	 * Inserts the Instagram data into the slider.
	 *
	 * @since 1.0.0
	 *
	 * @param array $data      Array of slider data.
	 * @param array $instagram Array of Instagram image data objects.
	 * @return array $data     Amended array of slider data.
	 */
	function insert_data( $data, $instagram ) {
	
	    // Empty out the current slider data.
	    $data['slider'] = array();
	
	    // Loop through and insert the Instagram data.
	    $instance = Soliloquy_Shortcode::get_instance();
	    foreach ( $instagram as $i => $image ) {
	        // Prepare variables.
	        $id             = ! empty( $image['id'] ) ? $image['id'] : $i;
	        $prep           = array();
	        $prep['status'] = 'active';
	        $prep['src']    = ! empty( $image['src'] ) ? esc_url( $image['src'] ) : '';
	        $prep['title']  = $prep['alt'] = $prep['caption'] = ! empty( $image['caption'] ) ? esc_attr( $image['caption'] ) : '';
	        $prep['link']   = ! empty( $image['link'] ) ? esc_url( $image['link'] ) : '';
	        $prep['type']   = 'image';
	
	        // Allow image to be filtered for each image.
	        $prep = apply_filters( 'soliloquy_instagram_image', $prep, $instagram, $data );
	
	        // Insert the image into the slider.
	        $data['slider'][$id] = $prep;
	    }
	
	    // Return and allow filtering of final data.
	    return apply_filters( 'soliloquy_instagram_slider_data', $data, $instagram );
	
	}
	
	/**
	 * Adds a custom slider class to denote an Instagram slider.
	 *
	 * @since 1.0.0
	 *
	 * @param array $classes  Array of slider classes.
	 * @param array $data     Array of slider data.
	 * @return array $classes Amended array of slider classes.
	 */
	function output_class( $classes, $data ) {
	
	    // Return early if not an Instagram slider.
	    $instance = Soliloquy_Shortcode::get_instance();
	    if ( 'instagram' !== $instance->get_config( 'type', $data ) ) {
	        return $classes;
	    }
	
	    // Add custom Instagram class.
	    $classes[] = 'soliloquy-instagram-slider';
	    return $classes;
	
	}	
	
    /**
     * Returns the singleton instance of the class.
     *
     * @since 2.2.0
     *
     * @return object The Soliloquy_Thumbnails_Shortcode object.
     */
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Soliloquy_Instagram_Shortcode ) ) {
            self::$instance = new Soliloquy_Instagram_Shortcode();
        }

        return self::$instance;

    }	
}

$soliloquy_instagram_shortcode = Soliloquy_Instagram_Shortcode::get_instance();