<?php
/**
 * Plugin Name: Soliloquy - Instagram Addon
 * Plugin URI:  http://soliloquywp.com
 * Description: Enables Instagram sliders in Soliloquy.
 * Author:      Soliloquy Team
 * Author URI:  http://soliloquywp.com
 * Version:     2.2.2
 * Text Domain: soliloquy-instagram
 * Domain Path: languages
 *
 * Soliloquy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * Soliloquy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Soliloquy. If not, see <http://www.gnu.org/licenses/>.
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}
class Soliloquy_Instagram{
   /**
     * Holds the class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public static $instance;

    /**
     * Plugin version, used for cache-busting of style and script file references.
     *
     * @since 2.2.0
     *
     * @var string
     */
    public $version = '2.2.2';

    /**
     * The name of the plugin.
     *
     * @since 2.2.0
     *
     * @var string
     */
    public $plugin_name = 'Soliloquy - Instagram Addon';

    /**
     * Unique plugin slug identifier.
     *
     * @since 2.2.0
     *
     * @var string
     */
    public $plugin_slug = 'soliloquy-instagram';

    /**
     * Plugin file.
     *
     * @since 2.2.0
     *
     * @var string
     */
    public $file = __FILE__;

    /**
     * Primary class constructor.
     *
     * @since 2.2.0
     */
    public function __construct(){

	    // Load the plugin textdomain.
        add_action( 'plugins_loaded', array( $this, 'load_plugin_textdomain' ) );

        // Load the plugin.
        add_action( 'soliloquy_init', array( $this, 'init' ), 99 );

    }

    /**
     * Loads the plugin textdomain for translation.
     *
     * @since 2.2.0
     */
    public function load_plugin_textdomain() {

        load_plugin_textdomain( $this->plugin_slug, false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );

    }

    public function init(){

        // Load admin only components.
        if ( is_admin() ) {

        	$this->require_admin();

        }

        // Load global components.
        $this->require_global();

        // Load the updater
        add_action( 'soliloquy_updater', array( $this, 'updater' ) );

    }


    /**
     * Loads all admin related files into scope.
     *
     * @since 2.2.0
     */
    public function require_admin() {

	   	require plugin_dir_path( __FILE__ ) . 'includes/admin/common.php';
        require plugin_dir_path( __FILE__ ) . 'includes/admin/metaboxes.php';
        require plugin_dir_path( __FILE__ ) . 'includes/admin/settings.php';

    }

    /**
	 * Initializes the addon updater.
	 *
     * @since 2.2.0
	 *
	 * @param string $key The user license key.
	 */
	function updater( $key ) {

	    $args = array(
	        'plugin_name' => $this->plugin_name,
	        'plugin_slug' => $this->plugin_slug,
	        'plugin_path' => plugin_basename( __FILE__ ),
	        'plugin_url'  => trailingslashit( WP_PLUGIN_URL ) . $this->plugin_slug,
	        'remote_url'  => 'http://soliloquywp.com/',
	        'version'     => $this->version,
	        'key'         => $key,
	    );

	    $this->soliloquy_instagram = new Soliloquy_Updater( $args );

	}

	/**
	 * Returns the default options for the Instagram addon.
	 *
	 * @since 1.0.0
	 *
	 * @return array Array of default options.
	 */
	function default_options() {

	    return array(
	        'token' => '',
	        'id'    => 0
	    );

	}

    /**
     * Loads all global files into scope.
     *
     * @since 2.2.0
     */
    public function require_global() {
	   	require_once plugin_dir_path( __FILE__ ) . 'includes/admin/common.php';
        require plugin_dir_path( __FILE__ ) . 'includes/global/shortcode.php';

    }
    function activation( $network_wide ){

	    $instance = Soliloquy::get_instance();
		$common = Soliloquy_Instagram_Common::get_instance();
	    if ( is_multisite() && $network_wide ) {
	        global $wpdb;
	        $site_list = $wpdb->get_results( "SELECT * FROM $wpdb->blogs ORDER BY blog_id" );
	        foreach ( (array) $site_list as $site ) {
	            switch_to_blog( $site->blog_id );

	            // Set default option.
	            $option = get_option( 'soliloquy_instagram' );
	            if ( ! $option || empty( $option ) ) {
	                update_option( 'soliloquy_instagram', $common->default_options() );
	            }

	            restore_current_blog();
	        }
	    } else {
	        // Set default option.
	        $option = get_option( 'soliloquy_instagram' );
	        if ( ! $option || empty( $option ) ) {
	            update_option( 'soliloquy_instagram', $common->default_options() );
	        }
	    }

    }

    function deactivation(){
	    if ( is_multisite() ) {
	        global $wpdb;
	        $site_list = $wpdb->get_results( "SELECT * FROM $wpdb->blogs ORDER BY blog_id" );
	        foreach ( (array) $site_list as $site ) {
	            switch_to_blog( $site->blog_id );
	            delete_option( 'soliloquy_instagram' );
	            restore_current_blog();
	        }
	    } else {
	        delete_option( 'soliloquy_instagram' );
	    }

    }

    /**
     * Returns the singleton instance of the class.
     *
     * @since 2.2.0
     *
     * @return object The Soliloquy object.
     */
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Soliloquy_Instagram ) ) {
            self::$instance = new Soliloquy_Instagram();
        }

        return self::$instance;

    }


}

$soliloquy_instagram = Soliloquy_Instagram::get_instance();

//DELETE EVERYTHING BELOW HERE!!!

// Register activation and uninstall hooks.
register_activation_hook( __FILE__, 'soliloquy_instagram_activation_hook' );
/**
 * Fired when the plugin is activated.
 *
 * @since 1.0.0
 *
 * @global object $wpdb         The WordPress database object.
 * @param boolean $network_wide True if WPMU superadmin uses "Network Activate" action, false otherwise.
 */
function soliloquy_instagram_activation_hook( $network_wide ) {

    $instance = Soliloquy_Instagram::get_instance();

    if ( is_multisite() && $network_wide ) {
        global $wpdb;
        $site_list = $wpdb->get_results( "SELECT * FROM $wpdb->blogs ORDER BY blog_id" );
        foreach ( (array) $site_list as $site ) {
            switch_to_blog( $site->blog_id );

            // Set default option.
            $option = get_option( 'soliloquy_instagram' );
            if ( ! $option || empty( $option ) ) {
                update_option( 'soliloquy_instagram', $instance->default_options() );
            }

            restore_current_blog();
        }
    } else {
        // Set default option.
        $option = get_option( 'soliloquy_instagram' );
        if ( ! $option || empty( $option ) ) {
            update_option( 'soliloquy_instagram', $instance->default_options() );
        }
    }

}

register_uninstall_hook( __FILE__, 'soliloquy_instagram_uninstall_hook' );
/**
 * Fired when the plugin is uninstalled.
 *
 * @since 1.0.0
 *
 * @global object $wpdb The WordPress database object.
 */
function soliloquy_instagram_uninstall_hook() {

    $instance = Soliloquy_Instagram::get_instance();

    if ( is_multisite() ) {
        global $wpdb;
        $site_list = $wpdb->get_results( "SELECT * FROM $wpdb->blogs ORDER BY blog_id" );
        foreach ( (array) $site_list as $site ) {
            switch_to_blog( $site->blog_id );
            delete_option( 'soliloquy_instagram' );
            restore_current_blog();
        }
    } else {
        delete_option( 'soliloquy_instagram' );
    }

}
