<?php
	
class Soliloquy_Pinterest_Metaboxes{

    /**
     * Holds the class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public static $instance;

    /**
     * Path to the file.
     *
     * @since 2.2.0
     *
     * @var string
     */
    public $file = __FILE__;

    /**
     * Holds the base class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public $base;
    /**
     * Primary class constructor.
     *
     * @since 2.2.0
     */
    public function __construct() {

    	// Get base instance
    	$this->base = Soliloquy_Pinterest::get_instance();	
  	  	
		add_filter( 'soliloquy_defaults', array( $this, 'defaults' ), 10, 2 );
		add_filter( 'soliloquy_tab_nav', array( $this, 'tab_nav' ) );
		add_action( 'soliloquy_tab_pinterest', array( $this, 'pinterest_settings' ) );
		add_action( 'soliloquy_lightbox_box', array( $this, 'lightbox_settings' ) );
		add_filter( 'soliloquy_save_settings', array( $this, 'save' ), 10, 2 );
		
    add_filter( 'soliloquy_output_start', 'soliloquy_pinterest_style', 10, 2 );
    add_filter( 'soliloquy_output_start', 'soliloquy_pinterest_lightbox_style', 10, 2 );
    add_filter( 'soliloquy_output_link_attr', 'soliloquy_pinterest_lightbox_attr', 10, 5 );
    add_filter( 'soliloquy_output_after_link', 'soliloquy_pinterest_output', 10, 4 );
    add_action( 'soliloquy_api_end', 'soliloquy_pinterest_event' );
    add_action( 'soliloquy_lightbox_api_before_show', 'soliloquy_pinterest_lightbox' );
	
	}
	
	/**
	 * Applies a default to the addon setting.
	 *
	 * @since 1.0.0
	 *
	 * @param array $defaults  Array of default config values.
	 * @param int $post_id     The current post ID.
	 * @return array $defaults Amended array of default config values.
	 */
	function defaults( $defaults, $post_id ) {
	
	    // Pinterest addon defaults.
	    $defaults['pinterest']                   = 0;
	    $defaults['pinterest_position']          = 'top_left';
	    $defaults['pinterest_color']             = 'gray';
	    $defaults['lightbox_pinterest']          = 0;
	    $defaults['lightbox_pinterest_position'] = 'top_left';
	    $defaults['lightbox_pinterest_color']    = 'gray';
	    return $defaults;
	
	}
	
	/**
	 * Filters in a new tab for the addon.
	 *
	 * @since 1.0.0
	 *
	 * @param array $tabs  Array of default tab values.
	 * @return array $tabs Amended array of default tab values.
	 */
	function tab_nav( $tabs ) {
	
	    $tabs['pinterest'] = __( 'Pinterest', 'soliloquy-pinterest' );
	    return $tabs;
	
	}
	
	/**
	 * Adds addon setting to the Pinterest tab.
	 *
	 * @since 1.0.0
	 *
	 * @param object $post The current post object.
	 */
	function pinterest_settings( $post ) {
	
	    $instance = Soliloquy_Metaboxes::get_instance();
		$common = Soliloquy_Pinterest_Admin_Common::get_instance();
	    ?>
	    <div id="soliloquy-pinterest">
		    <div class="soliloquy-config-header">
	        	<h2 class="soliloquy-intro"><?php _e( 'The settings below adjust the Pinterest settings for the slider.', 'soliloquy-pinterest' ); ?></h2>
				<p class="soliloquy-help"><?php _e( 'Need some help?', 'soliloquy' ); ?><a href="http://soliloquywp.com/docs/pinterest-addon/" target="_blank"><?php _e( ' Watch a video on how to setup your slider configuration', 'soliloquy-lightbox' ); ?></a></p>
			</div>	  	    
	        <table class="form-table">
	            <tbody>
	                <tr id="soliloquy-config-pinterest-box">
	                    <th scope="row">
	                        <label for="soliloquy-config-pinterest"><?php _e( 'Enable Pin It Button?', 'soliloquy-pinterest' ); ?></label>
	                    </th>
	                    <td>
	                        <input id="soliloquy-config-pinterest" type="checkbox" name="_soliloquy[pinterest]" value="<?php echo $instance->get_config( 'pinterest', $instance->get_config_default( 'pinterest' ) ); ?>" <?php checked( $instance->get_config( 'pinterest', $instance->get_config_default( 'pinterest' ) ), 1 ); ?> />
	                        <span class="description"><?php _e( 'Enables or disables the Pinterest "Pin It" button for slider images.', 'soliloquy-pinterest' ); ?></span>
	                    </td>
	                </tr>
	                <tr id="soliloquy-config-pinterest-position-box">
	                    <th scope="row">
	                        <label for="soliloquy-config-pinterest-position"><?php _e( 'Pinterest Position', 'soliloquy-pinterest' ); ?></label>
	                    </th>
	                    <td>
		                    <div class="soliloquy-select">
	                        <select id="soliloquy-config-pinterest-position" name="_soliloquy[pinterest_position]" class="soliloquy-chosen" data-soliloquy-chosen-options='{ "disable_search":"true", "width": "100%" }'>
	                            <?php foreach ( (array) $common->pinterest_positions() as $i => $data ) : ?>
	                                <option value="<?php echo $data['value']; ?>"<?php selected( $data['value'], $instance->get_config( 'pinterest_position', $instance->get_config_default( 'pinterest_position' ) ) ); ?>><?php echo $data['name']; ?></option>
	                            <?php endforeach; ?>
	                        </select>
		                    </div>
	                        <p class="description"><?php _e( 'Sets the position of the Pinterest button on the slider images.', 'soliloquy-pinterest' ); ?></p>
	                    </td>
	                </tr>
	                <tr id="soliloquy-config-pinterest-color-box">
	                    <th scope="row">
	                        <label for="soliloquy-config-pinterest-color"><?php _e( 'Pinterest Button Color', 'soliloquy-pinterest' ); ?></label>
	                    </th>
	                    <td>
		                    <div class="soliloquy-select">
	                        <select id="soliloquy-config-pinterest-color" name="_soliloquy[pinterest_color]" class="soliloquy-chosen" data-soliloquy-chosen-options='{ "disable_search":"true", "width": "100%" }'>
	                            <?php foreach ( (array) $common->pinterest_colors() as $i => $data ) : ?>
	                                <option value="<?php echo $data['value']; ?>"<?php selected( $data['value'], $instance->get_config( 'pinterest_color', $instance->get_config_default( 'pinterest_color' ) ) ); ?>><?php echo $data['name']; ?></option>
	                            <?php endforeach; ?>
	                        </select>
		                    </div>
	                        <p class="description"><?php _e( 'Sets the color of the Pin It button.', 'soliloquy-pinterest' ); ?></p>
	                    </td>
	                </tr>
	            </tbody>
	        </table>
	    </div>
	    <?php
	
	}
	
	/**
	 * Adds addon setting to the Lightbox tab.
	 *
	 * @since 1.0.0
	 *
	 * @param object $post The current post object.
	 */
	function lightbox_settings( $post ) {
	
	    $instance = Soliloquy_Metaboxes::get_instance();
	    $common = Soliloquy_Pinterest_Admin_Common::get_instance();
	    ?>
	    <tr id="soliloquy-config-lightbox-pinterest-box">
	        <th scope="row">
	            <label for="soliloquy-config-lightbox-pinterest"><?php _e( 'Enable Pin It Button in Lightbox?', 'soliloquy-pinterest' ); ?></label>
	        </th>
	        <td>
	            <input id="soliloquy-config-lightbox-pinterest" type="checkbox" name="_soliloquy[lightbox_pinterest]" value="<?php echo $instance->get_config( 'lightbox_pinterest', $instance->get_config_default( 'lightbox_pinterest' ) ); ?>" <?php checked( $instance->get_config( 'lightbox_pinterest', $instance->get_config_default( 'lightbox_pinterest' ) ), 1 ); ?> />
	            <span class="description"><?php _e( 'Enables or disables the Pinterest "Pin It" button for lightbox images.', 'soliloquy-pinterest' ); ?></span>
	        </td>
	    </tr>
	    <tr id="soliloquy-config-lightbox-pinterest-position-box">
	        <th scope="row">
	            <label for="soliloquy-config-lightbox-pinterest-position"><?php _e( 'Pinterest Position', 'soliloquy-pinterest' ); ?></label>
	        </th>
	        <td>
		        <div class="soliloquy-select">
	            <select id="soliloquy-config-lightbox-pinterest-position" name="_soliloquy[lightbox_pinterest_position]" class="soliloquy-chosen" data-soliloquy-chosen-options='{ "disable_search":"true", "width": "100%" }'>
	                <?php foreach ( (array) $common->pinterest_positions() as $i => $data ) : ?>
	                    <option value="<?php echo $data['value']; ?>"<?php selected( $data['value'], $instance->get_config( 'lightbox_pinterest_position', $instance->get_config_default( 'lightbox_pinterest_position' ) ) ); ?>><?php echo $data['name']; ?></option>
	                <?php endforeach; ?>
	            </select>
		        </div>
	            <p class="description"><?php _e( 'Sets the position of the Pinterest button on the lightbox images.', 'soliloquy-pinterest' ); ?></p>
	        </td>
	    </tr>
	    <tr id="soliloquy-config-lightbox-pinterest-color-box">
	        <th scope="row">
	            <label for="soliloquy-config-lightbox-pinterest-color"><?php _e( 'Pinterest Button Color', 'soliloquy-pinterest' ); ?></label>
	        </th>
	        <td>
		        <div class="soliloquy-select">
	            <select id="soliloquy-config-lightbox-pinterest-color" name="_soliloquy[lightbox_pinterest_color]" class="soliloquy-chosen" data-soliloquy-chosen-options='{ "disable_search":"true", "width": "100%" }'>
	                <?php foreach ( (array) $common->pinterest_colors() as $i => $data ) : ?>
	                    <option value="<?php echo $data['value']; ?>"<?php selected( $data['value'], $instance->get_config( 'lightbox_pinterest_color', $instance->get_config_default( 'lightbox_pinterest_color' ) ) ); ?>><?php echo $data['name']; ?></option>
	                <?php endforeach; ?>
	            </select>
		        </div>
	            <p class="description"><?php _e( 'Sets the color of the Pin It button in the lightbox.', 'soliloquy-pinterest' ); ?></p>
	        </td>
	    </tr>
	    <?php
	
	}
	
	/**
	 * Saves the addon setting.
	 *
	 * @since 1.0.0
	 *
	 * @param array $settings  Array of settings to be saved.
	 * @param int $pos_tid     The current post ID.
	 * @return array $settings Amended array of settings to be saved.
	 */
	function save( $settings, $post_id ) {
	
	    $settings['config']['pinterest']                   = isset( $_POST['_soliloquy']['pinterest'] ) ? 1 : 0;
	    $settings['config']['pinterest_position']          = esc_attr( $_POST['_soliloquy']['pinterest_position'] );
	    $settings['config']['pinterest_color']             = esc_attr( $_POST['_soliloquy']['pinterest_color'] );
	    
	    // Only save if Lightbox Addon exists
	    $settings['config']['lightbox_pinterest']          = isset( $_POST['_soliloquy']['lightbox_pinterest'] ) ? 1 : 0;
	    $settings['config']['lightbox_pinterest_position'] = ( isset( $_POST['_soliloquy']['lightbox_pinterest_position'] ) ? esc_attr( $_POST['_soliloquy']['lightbox_pinterest_position'] ) : '' );
	    $settings['config']['lightbox_pinterest_color']    = ( isset( $_POST['_soliloquy']['lightbox_pinterest_color'] ) ? esc_attr( $_POST['_soliloquy']['lightbox_pinterest_color'] ) : '' );
	
	    return $settings;
	
	}	
	/**
     * Returns the singleton instance of the class.
     *
     * @since 2.2.0
     *
     * @return object The Soliloquy_Pinterest_Metaboxes object.
     */
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Soliloquy_Pinterest_Metaboxes ) ) {
            self::$instance = new Soliloquy_Pinterest_Metaboxes();
        }

        return self::$instance;

    }   	 
    	
}	

$soliloquy_pinterest_metaboxes = Soliloquy_Pinterest_Metaboxes::get_instance();