<?php
	
class Soliloquy_Pinterest_Shortcode{
    /**
     * Holds the class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public static $instance;

    /**
     * Path to the file.
     *
     * @since 2.2.0
     *
     * @var string
     */
    public $file = __FILE__;

    /**
     * Holds the base class object.
     *
     * @since 2.2.0
     *
     * @var object
     */
    public $base;

    /**
     * Primary class constructor.
     *
     * @since 2.2.0
     */	
	function __construct(){
	
		$this->base = Soliloquy_Pinterest::get_instance();
		
		add_filter( 'soliloquy_output_start', array( $this, 'style' ), 10, 2 );
		add_filter( 'soliloquy_output_start', array( $this, 'lightbox_style' ), 10, 2 );
		add_filter( 'soliloquy_output_link_attr', array( $this, 'lightbox_attr' ), 10, 5 );
		add_filter( 'soliloquy_output_after_link', array( $this, 'output' ), 10, 4 );
	    add_action( 'soliloquy_api_end', array( $this, 'event' ) );
	    add_action( 'soliloquy_lightbox_api_before_show', array( $this, 'lightbox' ) );
		
	}
	
	/**
	 * Outputs the Pinterest button styles.
	 *
	 * @since 1.0.0
	 *
	 * @param string $output  The slider HTML output.
	 * @param array $data     Array of slider data.
	 * @return string $output Amended slider HTML output.
	 */
	function style( $output, $data ) {
	
	    $instance = Soliloquy_Shortcode::get_instance();
	    if ( ! $instance->get_config( 'pinterest', $data ) ) {
	        return $output;
	    }
	
	    // Since this CSS only needs to be defined once on a page, use static flag to help keep track.
	    static $soliloquy_pinterest_css_flag = false;
	
	    // If the tag has been set to true, return the default output.
	    if ( $soliloquy_pinterest_css_flag ) {
	        return $output;
	    }
	
	    // Build out our custom CSS.
	    $css  = '<style type="text/css">';
	        // Apply a base reset for all items in the filter list to avoid as many conflicts as possible.
	        $css .= '.soliloquy-container .soliloquy-pinterest-share { background-color: transparent; transition: none; -moz-transition: none; -webkit-transition: none; }';
	        $css .= '.soliloquy-container .soliloquy-pinterest-share:hover { background-position: 0 -28px; }';
	        $css .= '.soliloquy-container .soliloquy-pinterest-share:active { background-position: 0 -56px; }';
	        $css .= '.soliloquy-container .soliloquy-pinterest-share.soliloquy-pinterest-gray { background-image: url(' . plugins_url( 'assets/images/pinterest-gray.png', $this->base->file ) . '); }';
	        $css .= '.soliloquy-container .soliloquy-pinterest-share.soliloquy-pinterest-red { background-image: url(' . plugins_url( 'assets/images/pinterest-red.png', $this->base->file ) . '); }';
	        $css .= '.soliloquy-container .soliloquy-pinterest-share.soliloquy-pinterest-white { background-image: url(' . plugins_url( 'assets/images/pinterest-white.png', $this->base->file ) . '); }';
	        $css .= '@media only screen and (-webkit-min-device-pixel-ratio: 2),only screen and (min--moz-device-pixel-ratio: 2),only screen and (-o-min-device-pixel-ratio: 2/1), only screen and (min-device-pixel-ratio: 2),only screen and (min-resolution: 192dpi),only screen and (min-resolution: 2dppx) {';
	            $css .= '.soliloquy-container .soliloquy-pinterest-share { background-size: 56px 84px; }';
	            $css .= '.soliloquy-container .soliloquy-pinterest-share.soliloquy-pinterest-gray { background-image: url(' . plugins_url( 'assets/images/pinterest-gray@2x.png', $this->base->file ) . '); }';
	            $css .= '.soliloquy-container .soliloquy-pinterest-share.soliloquy-pinterest-red { background-image: url(' . plugins_url( 'assets/images/pinterest-red@2x.png', $this->base->file ) . '); }';
	            $css .= '.soliloquy-container .soliloquy-pinterest-share.soliloquy-pinterest-white { background-image: url(' . plugins_url( 'assets/images/pinterest-white@2x.png', $this->base->file ) . '); }';
	        $css .= '}';
	    $css .= '</style>';
	
	    // Set our flag to true.
	    $soliloquy_pinterest_css_flag = true;
	
	    // Return the minified CSS.
	    $minify = $instance->minify( $css );
	    return $css . $output;
	
	}
	
	/**
	 * Outputs the Pinterest lightbox button styles.
	 *
	 * @since 1.0.0
	 *
	 * @param string $output  The slider HTML output.
	 * @param array $data     Array of slider data.
	 * @return string $output Amended slider HTML output.
	 */
	function lightbox_style( $output, $data ) {
	
	    $instance = Soliloquy_Shortcode::get_instance();
	    if ( ! $instance->get_config( 'lightbox_pinterest', $data ) ) {
	        return $output;
	    }
	
	    // Since this CSS only needs to be defined once on a page, use static flag to help keep track.
	    static $soliloquy_pinterest_lightbox_css_flag = false;
	
	    // If the tag has been set to true, return the default output.
	    if ( $soliloquy_pinterest_lightbox_css_flag ) {
	        return $output;
	    }
	
	    // Build out our custom CSS.
	    $css  = '<style type="text/css">';
	        // Apply a base reset for all items in the filter list to avoid as many conflicts as possible.
	        $css .= '.soliloquybox-wrap .soliloquy-pinterest-share { background-color: transparent; transition: none; -moz-transition: none; -webkit-transition: none; }';
	        $css .= '.soliloquybox-wrap .soliloquy-pinterest-share:hover { background-position: 0 -28px; }';
	        $css .= '.soliloquybox-wrap .soliloquy-pinterest-share:active { background-position: 0 -56px; }';
	        $css .= '.soliloquybox-wrap .soliloquy-pinterest-share.soliloquy-pinterest-gray { background-image: url(' . plugins_url( 'assets/images/pinterest-gray.png', $this->base->file ) . '); }';
	        $css .= '.soliloquybox-wrap .soliloquy-pinterest-share.soliloquy-pinterest-red { background-image: url(' . plugins_url( 'assets/images/pinterest-red.png', $this->base->file ) . '); }';
	        $css .= '.soliloquybox-wrap .soliloquy-pinterest-share.soliloquy-pinterest-white { background-image: url(' . plugins_url( 'assets/images/pinterest-white.png', $this->base->file ) . '); }';
	        $css .= '@media only screen and (-webkit-min-device-pixel-ratio: 2),only screen and (min--moz-device-pixel-ratio: 2),only screen and (-o-min-device-pixel-ratio: 2/1), only screen and (min-device-pixel-ratio: 2),only screen and (min-resolution: 192dpi),only screen and (min-resolution: 2dppx) {';
	            $css .= '.soliloquybox-wrap .soliloquy-pinterest-share { background-size: 56px 84px; }';
	            $css .= '.soliloquybox-wrap .soliloquy-pinterest-share.soliloquy-pinterest-gray { background-image: url(' . plugins_url( 'assets/images/pinterest-gray@2x.png', $this->base->file ) . '); }';
	            $css .= '.soliloquybox-wrap .soliloquy-pinterest-share.soliloquy-pinterest-red { background-image: url(' . plugins_url( 'assets/images/pinterest-red@2x.png', $this->base->file ) . '); }';
	            $css .= '.soliloquybox-wrap .soliloquy-pinterest-share.soliloquy-pinterest-white { background-image: url(' . plugins_url( 'assets/images/pinterest-white@2x.png', $this->base->file ) . '); }';
	        $css .= '}';
	    $css .= '</style>';
	
	    // Set our flag to true.
	    $soliloquy_pinterest_lightbox_css_flag = true;
	
	    // Return the minified CSS.
	    $minify = $instance->minify( $css );
	    return $css . $output;
	
	}
	
	/**
	 * Adds the proper attributes to assets/images for Pinterest output in the lightbox.
	 *
	 * @since 1.0.0
	 *
	 * @param string $attr  String of link attributes.
	 * @param int $id       The current slider ID.
	 * @param array $item   Array of slide data.
	 * @param array $data   Array of slider data.
	 * @param int $i        The current position in the slider.
	 * @return string $attr Amended string of link attributes.
	 */
	function lightbox_attr( $attr, $id, $item, $data, $i ) {
	
	    // If there is no lightbox, don't output anything.
	    $instance = Soliloquy_Shortcode::get_instance();
	    if ( ! $instance->get_config( 'lightbox_pinterest', $data ) ) {
	        return $attr;
	    }
	
	    // If the $post variable is not set, set the URL to the home page of the site.
	    global $post;
	    if ( isset( $post ) ) {
	        $url = get_permalink( $post->ID );
	    } else {
	        $url = trailingslashit( get_home_url() );
	    }
	    $url = apply_filters( 'soliloquy_pinterest_url', $url, $id, $item, $data );
	
	    // Set the style for the Pin It button.
	    $style = '';
	    switch ( $instance->get_config( 'lightbox_pinterest_position', $data ) ) {
	        case 'top_left' :
	        default :
	            $style = 'top:10px;left:10px;';
	            break;
	        case 'top_right' :
	            $style = 'top:10px;right:10px;';
	            break;
	        case 'bottom_right' :
	            $style = 'bottom:10px;right:10px;';
	            break;
	        case 'bottom_left' :
	            $style = 'bottom:10px;left:10px;';
	            break;
	    }
	
	    // Set the description for the image.
	    $title       = ! empty( $item['caption'] ) ? $item['caption'] : $item['title'];
	    $description = apply_filters( 'soliloquy_pinterest_description', $title, $id, $item, $data );
	
	    // Append the button to the image with styles.
	    $output = apply_filters( 'soliloquy_pinterest_output', '<a class="soliloquy-pinterest-share soliloquy-pinterest-' . $instance->get_config( 'lightbox_pinterest_color', $data ) . '" href="http://pinterest.com/pin/create/button/?url=' . esc_url( $url ) . '&description=' . urlencode( strip_tags( $description ) ) . '&media=' . esc_url( $item['src'] ) . '" rel="nofollow" style="width:56px;height:28px;display:block;outline:none;position:absolute;z-index:9999999;' . $style . '"></a>', $id, $item, $data );
	    $attr .= ' data-soliloquy-pinterest="' . esc_attr( $output ) . '"';
	
	    return apply_filters( 'soliloquy_pinterest_lightbox_attr', $attr, $id, $item, $data, $i );
	
	}
	
	/**
	 * Outputs the custom Pinterest to the specific slider.
	 *
	 * @since 1.0.0
	 *
	 * @global object $post   The current post object.
	 * @param string $output  The HTML output for the slider.
	 * @param int $id         The slider ID.
	 * @param array $item     Array of data about the image.
	 * @param array $data     Data for the slider.
	 * @return string $output Amended slider HTML.
	 */
	function output( $output, $id, $item, $data ) {
	
	    // If there is no Pinterest button, return the default slider HTML.
	    $instance = Soliloquy_Shortcode::get_instance();
	    if ( ! $instance->get_config( 'pinterest', $data ) ) {
	        return $output;
	    }
	
	    // If the $post variable is not set, set the URL to the home page of the site.
	    global $post;
	    if ( isset( $post ) ) {
	        $url = get_permalink( $post->ID );
	    } else {
	        $url = trailingslashit( get_home_url() );
	    }
	    $url = apply_filters( 'soliloquy_pinterest_url', $url, $id, $item, $data );
	
	    // Set the style for the Pin It button.
	    $style = '';
	    switch ( $instance->get_config( 'pinterest_position', $data ) ) {
	        case 'top_left' :
	        default :
	            $style = 'top:10px;left:10px;';
	            break;
	        case 'top_right' :
	            $style = 'top:10px;right:10px;';
	            break;
	        case 'bottom_right' :
	            $style = 'bottom:10px;right:10px;';
	            break;
	        case 'bottom_left' :
	            $style = 'bottom:10px;left:10px;';
	            break;
	    }
	
	    // Set the description for the image.
	    $title       = ! empty( $item['caption'] ) ? $item['caption'] : $item['title'];
	    $description = apply_filters( 'soliloquy_pinterest_description', $title, $id, $item, $data );
	
	    // Append the button to the image with styles.
	    $output .= apply_filters( 'soliloquy_pinterest_output', '<a class="soliloquy-pinterest-share soliloquy-pinterest-' . $instance->get_config( 'pinterest_color', $data ) . '" href="http://pinterest.com/pin/create/button/?url=' . esc_url( $url ) . '&description=' . urlencode( strip_tags( $description ) ) . '&media=' . esc_url( $item['src'] ) . '" rel="nofollow" style="width:56px;height:28px;display:block;outline:none;position:absolute;z-index:99999;' . $style . '"></a>', $id, $item, $data );
	
	    // Return the amended HTML.
	    return $output;
	
	}
	
	/**
	 * Output the JS to have the button click open in a new window.
	 *
	 * @since 1.0.0
	 *
	 * @param array $data Data for the slider.
	 */
	function event( $data ) {
	
	    // If there is no Pinterest button, do nothing.
	    $instance = Soliloquy_Shortcode::get_instance();
	    if ( ! $instance->get_config( 'pinterest', $data ) ) {
	        return;
	    }
	
	    // Output JS to open button click in a new window.
	    ob_start();
	    ?>
	    $(document).on('click', '.soliloquy-pinterest-share', function(e){
	        e.preventDefault();
	        window.open($(this).attr('href'), 'soliloquy-pinterest', 'menubar=1,resizable=1,width=760,height=360');
	    });
	    <?php
	    echo ob_get_clean();
	
	}
	
	/**
	 * Enables Pinterest inside lightboxes if it is enabled.
	 *
	 * @since 1.0.0
	 *
	 * @param array $data Data for the gallery.
	 */
	function lightbox( $data ) {
	
	    // If there is no Pinterest button for the lightbox, do nothing.
	    $instance = Soliloquy_Shortcode::get_instance();
	    if ( ! $instance->get_config( 'lightbox_pinterest', $data ) ) {
	        return;
	    }
	
	    ob_start();
	    ?>
	    if ( $(this.element).data('soliloquy-pinterest') ) {
	        $(this.inner).append($(this.element).data('soliloquy-pinterest'));
	    }
	    <?php
	    echo ob_get_clean();
	
	}
		
    /**
     * Returns the singleton instance of the class.
     *
     * @since 2.2.0
     *
     * @return object The Soliloquy_Thumbnails_Shortcode object.
     */
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Soliloquy_Pinterest_Shortcode ) ) {
            self::$instance = new Soliloquy_Pinterest_Shortcode();
        }

        return self::$instance;

    }	
}

$soliloquy_pinterest_shortcode = Soliloquy_Pinterest_Shortcode::get_instance();